#pragma once

#ifndef  WIN32_LEAN_AND_MEAN
#define WIN32_LEAN_AND_MEAN
#endif

#include <Windows.h>

#define NOMINMAX
#define WIN32_LEAN_AND_MEAN
#include <windows.h>
#define _USE_MATH_DEFINES
#include <math.h>
#define GLM_FORCE_CTOR_INIT
#include <glm/glm.hpp>
#define GLM_ENABLE_EXPERIMENTAL
#include <glm/gtx/transform.hpp>
#include <glm/gtx/euler_angles.hpp>

#include <string>
#include <d3d12.h>
#include <d3dx12.h>
#include <d3dcompiler.h>
#include <comdef.h>
#include <dxgi1_6.h>
#include <dxgiformat.h>
#include <fstream>
#include <dxcapi.use.h>
#include <vector>
#include <array>

#include <memory>
#include <functional>
#include <string>
#include <vector>

#include <imgui.h>
#include <backends\imgui_impl_dx12.h>
#include <backends\imgui_impl_win32.h>

static constexpr const uint32_t DefaultSwapChainBuffers = 3;
static constexpr const float aspectRatio = 16.0f / 9.0f;

// Globals
constexpr const uint32_t NumUserDescriptorRanges = 16;
constexpr const uint32_t NumGlobalSRVDescriptorRanges = 7 + NumUserDescriptorRanges;
constexpr const uint32_t NumGlobalCBVDescriptorRanges = 7;

#define MAKE_SMART_COM_PTR(_a) _COM_SMARTPTR_TYPEDEF(_a, __uuidof(_a))
MAKE_SMART_COM_PTR(ID3D12Device5);
MAKE_SMART_COM_PTR(ID3D12GraphicsCommandList4);
MAKE_SMART_COM_PTR(ID3D12CommandQueue);
MAKE_SMART_COM_PTR(IDXGISwapChain3);
MAKE_SMART_COM_PTR(IDXGIFactory4);
MAKE_SMART_COM_PTR(IDXGIAdapter1);
MAKE_SMART_COM_PTR(ID3D12Fence);
MAKE_SMART_COM_PTR(ID3D12CommandAllocator);
MAKE_SMART_COM_PTR(ID3D12Resource);
MAKE_SMART_COM_PTR(ID3D12DescriptorHeap);
MAKE_SMART_COM_PTR(ID3D12Debug);
MAKE_SMART_COM_PTR(ID3D12StateObject);
MAKE_SMART_COM_PTR(ID3D12PipelineState);
MAKE_SMART_COM_PTR(ID3D12RootSignature);
MAKE_SMART_COM_PTR(ID3DBlob);
MAKE_SMART_COM_PTR(IDxcBlobEncoding);

template<typename T>
using UniquePtr = std::unique_ptr<T>;
template<typename T, typename ... Args>
constexpr UniquePtr<T> MakeUnique(Args&& ... args)
{
	return std::make_unique<T>(std::forward<Args>(args)...);
}

template<typename T>
using UniquePtrCustomDeleter = std::unique_ptr<T, std::function<void(T*)>>;

template<typename T>
using SharedPtr = std::shared_ptr<T>;
template<typename T, typename ... Args>
constexpr SharedPtr<T> MakeShared(Args&& ... args)
{
	return std::make_shared<T>(std::forward<Args>(args)...);
}

template<typename T>
using WeakPtr = std::weak_ptr<T>;
template<typename T, typename ... Args>
constexpr WeakPtr<T> MakeWeak(Args&& ... args)
{
	return std::weak_ptr<T>(std::forward<Args>(args)...);
}