#include "Application.h"
#include "Core/Exception.h"
#include "Layer.h"
#include "Rendering/Utils.h"

static bool showDemoWindow = false;

void ImGui_ImplWin32_InitPlatformInterface();

ImGuiLayer::ImGuiLayer(ID3D12DescriptorHeapPtr descHeap)
	:DescriptorHeap(descHeap)
{}

void ImGuiLayer::OnAttach(ID3D12Device5Ptr device)
{
	IMGUI_CHECKVERSION();
	ImGui::CreateContext();
	ImGuiIO& io = ImGui::GetIO();
	io.WantCaptureKeyboard = true;
	io.WantCaptureMouse = true;
	io.ConfigFlags |= ImGuiConfigFlags_NavEnableKeyboard;

	ImGui::StyleColorsDark();

	D3D12_DESCRIPTOR_HEAP_DESC heapDesc{};
	heapDesc.NumDescriptors = 1 + ((DescriptorHeap) ? DescriptorHeap->GetDesc().NumDescriptors : 0);
	heapDesc.Type = D3D12_DESCRIPTOR_HEAP_TYPE_CBV_SRV_UAV;
	heapDesc.Flags = D3D12_DESCRIPTOR_HEAP_FLAG_SHADER_VISIBLE;

	auto temp = DescriptorHeap.Detach();
	GRAPHICS_ASSERT(device->CreateDescriptorHeap(&heapDesc, IID_PPV_ARGS(&DescriptorHeap)));

	auto handle = DescriptorHeap->GetCPUDescriptorHandleForHeapStart();
	auto tempHandle = temp->GetCPUDescriptorHandleForHeapStart();
	handle.ptr += device->GetDescriptorHandleIncrementSize(D3D12_DESCRIPTOR_HEAP_TYPE_CBV_SRV_UAV);

	if(temp)
		device->CopyDescriptorsSimple(temp->GetDesc().NumDescriptors, handle, tempHandle, D3D12_DESCRIPTOR_HEAP_TYPE_CBV_SRV_UAV);

	ImGui_ImplWin32_Init(Application::GetApp().GetWindow()->GetHandle());
	ImGui_ImplDX12_Init(device, 
						DefaultSwapChainBuffers, 
						DXGI_FORMAT_R8G8B8A8_UNORM_SRGB,
						DescriptorHeap,
						DescriptorHeap->GetCPUDescriptorHandleForHeapStart(),
						DescriptorHeap->GetGPUDescriptorHandleForHeapStart());
}

void ImGuiLayer::OnDetach()
{
	ImGui_ImplDX12_Shutdown();
	ImGui_ImplWin32_Shutdown();
	ImGui::DestroyContext();
}

void ImGuiLayer::OnEvent(Event& e)
{
	ImGuiIO& io = ImGui::GetIO();
	e.Handled = e.GetCategory() == EventCategory::MouseEvents && io.WantCaptureMouse;
	e.Handled = e.GetCategory() == EventCategory::KeyEvents && io.WantCaptureKeyboard;
}

void ImGuiLayer::Begin()
{
	ImGui_ImplDX12_NewFrame();
	ImGui_ImplWin32_NewFrame();
	ImGui::NewFrame();
}

void ImGuiLayer::End(ID3D12GraphicsCommandList4Ptr cmdList)
{
	ImGuiIO& io = ImGui::GetIO();
	Application& app = Application::GetApp();
	io.DisplaySize = ImVec2((float)app.GetWindow()->GetWidth(), (float)app.GetWindow()->GetHeight());

	if (showDemoWindow)
		ImGui::ShowDemoWindow(&showDemoWindow);

	ImGui::Render();

	ID3D12DescriptorHeap* descriptorHeaps[] = { DescriptorHeap.GetInterfacePtr() };
	cmdList->SetDescriptorHeaps(_countof(descriptorHeaps), descriptorHeaps);

	ImGui_ImplDX12_RenderDrawData(ImGui::GetDrawData(), cmdList);

	if (io.ConfigFlags & ImGuiConfigFlags_ViewportsEnable)
	{
		ImGui::UpdatePlatformWindows();
		ImGui::RenderPlatformWindowsDefault();
	}
}